#include <stdarg.h>
#include <string.h>
#include "wwl.h"

/*?class WArgList

The class \typ{WArgList} encapsulates a standard X toolkit
\typ{ArgList}. It simplifies the handling of toolkit arglists and can
be used in two ways: either directly like arglists, whithout having to
worry about allocating enough room for arguments, or in conjunction
with member functions of wrapped widgets.

The latter is useful in cases where many resources have to be changed
at the same time, like constraint resources for the \typ{Form} widget.

Each member function modifying a resource for a widget class is
overloaded to permit the buffering of requests in a \typ{WArgList}
rather than modifying the widget it at once. For instance, instead of
writing:

\expandafter\ifx\csname indentation\endcsname\relax
\newlength{\indentation}\fi
\setlength{\indentation}{0.5em}
\begin{flushleft}
\hspace*{2\indentation}CXmForm cxf(MyButton);\mbox{}\\
\mbox{}\\
\hspace*{2\indentation}cxf.TopAttachment (XmATTACH_WIDGET);\mbox{}\\
\hspace*{2\indentation}cxf.TopWidget (MyOtherButton);\mbox{}\\
\end{flushleft}
which does not work usually because the form requires the
attachment and the widget attached to in the same call, one would
write instead:
\expandafter\ifx\csname indentation\endcsname\relax
\newlength{\indentation}\fi
\setlength{\indentation}{0.5em}
\begin{flushleft}
\hspace*{2\indentation}CXmForm cxf(MyButton);\mbox{}\\
\hspace*{2\indentation}WArgList args;\mbox{}\\
\mbox{}\\
\hspace*{2\indentation}cxf.TopAttachment (XmATTACH_WIDGET, args);\mbox{}\\
\hspace*{2\indentation}cxf.TopWidget (MyOtherButton, args);\mbox{}\\
\hspace*{2\indentation}cxf.Set (args);\mbox{}\\
\end{flushleft}

The same is true for setting a list in an \typ{XmList} widget. Both the
contents and the size of the table should be set at the same time.

\begin{em}
We are not satisfied with the syntax of buffered resource modification
propopsed here. It might change, though we will try to remain
compatible with this version. If you find a better mechanism or syntax
to buffer resource changes, please, let us know.
\end{em}

?*/

#ifdef DOC

/*?
Default constructor for \typ{WArgList}.
?*/
WArgList :: WArgList ()
{
	args = 0;
	num_args = alloc_args = 0;
}

/*?
Constructor for \typ{WArgList}. Preallocates a table of
resource/value, saves some time and space when the number of resource
is known. If more resources are specified than \var{Cardinal}, the
table grows.
?*/
WArgList :: WArgList (Cardinal len)
{
	args = new Arg[len];
	num_args = 0;
	alloc_args = len;
}

/*?
Constructor of \typ{WArgList} from a Xt arglist. Nothing is allocated
as longs as no resource is added. The arglist is reallocated in this
case.
?*/
inline
WArgList :: WArgList (ArgList a, Cardinal n)
{
	args = a;
	num_args = n;
	alloc_args = 0;
}
#endif

/*?
Vararg constructor for \typ{WArgList}. Beware to finish the argument
list with a NULL. Also, note that no checking is done. This is easier
to use and more readable in widget constructors than a static arglist.
?*/
WArgList :: WArgList (String name, XtArgVal value, ...)
{
	Cardinal n;
	String str;
	XtArgVal val;

	Add (name, value);
	va_list argv;
	va_start (argv, value);
	while (str = va_arg(argv, String))
		Add (str, va_arg (argv, XtArgVal));
	va_end (argv);
}

/*?nodoc?*/
Arg&
WArgList :: Grow (Cardinal n)
{
	if (n < alloc_args)
		return args[n];

	alloc_args = n + 16;
	ArgList new_args = new Arg[alloc_args];

	if (args && num_args)
		memcpy (new_args, args, num_args * sizeof (Arg));
	return args[n];
}


#ifdef DOC
/*?
Return the \typ{ArgList} contained in the \typ{WArgList}.
?*/
Arglist
WArgList :: Args () const
{
}

/*?
Return the number of valid arguments contained in the arglist.
?*/
Cardinal
WArgList :: NumArgs () const
{
}

/*?
Empty the \typ{WArgList}, make it ready to receive new resource/value
pairs.
?*/
void
WArgList :: Reset ()
{
}

/*?
Return the \var{n}th resource/value pair. Allocates it if required.
?*/
Arg&
WArgList :: operator [] (Cardinal n)
{
	if (n < num_args|| n < alloc_args)
		return args[n];
	else
		return Grow(n);
}

/*?nextdoc?*/
WArgList&
WArgList :: Add (String name, void* value)
{
	Arg&	a = (*this)[num_args];
	num_args++;
	a.name = name;
	a.value = (XtArgVal)value;
	return *this; 
}


/*?nextdoc?*/
WArgList&
WArgList :: Add (String name, const WXmString& value)
{
	return Add(name, (XmString)value);
}

/*?
Add a resource/value pair to the \typ{WArgList}.
?*/
WArgList&
WArgList :: Add (String name, long value)
{
	Arg&	a = (*this)[num_args];
	num_args++;
	a.name = name;
	a.value = (XtArgVal)value;
	return *this; 
}
#endif

#endif

